#
# raccp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2.
#

require 'racc/parser'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @racc = racc
      @interf    = racc.interf
      @ruletable = racc.ruletable
    end


    attr :classname

    attr :filename

    def parse( str, fname, mydebug = false )
      fname.must String
      
      @scanner = RaccScanner.new( str )
      @scanner.debug    = @yydebug  = mydebug
      @scanner.filename = @filename = fname

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      if val.respond_to? :id2name then
        v = val.id2name
      elsif String === val then
        v = val
      else
        v = val.inspect
      end
      raise ParseError,
%=parse error:#{@filename}:#{@scanner.lineno}: unexpected token '#{v}'=
    end

##### racc 1.1.0 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 6, 20, :_reduce_1,
 0, 21, :_reduce_none,
 2, 21, :_reduce_none,
 3, 23, :_reduce_4,
 1, 23, :_reduce_none,
 2, 23, :_reduce_6,
 2, 24, :_reduce_7,
 3, 24, :_reduce_8,
 3, 25, :_reduce_9,
 3, 25, :_reduce_10,
 1, 27, :_reduce_none,
 2, 27, :_reduce_none,
 2, 28, :_reduce_13,
 2, 28, :_reduce_14,
 2, 28, :_reduce_15,
 1, 29, :_reduce_16,
 2, 29, :_reduce_17,
 1, 26, :_reduce_18,
 1, 26, :_reduce_19,
 1, 22, :_reduce_20,
 1, 30, :_reduce_21,
 2, 30, :_reduce_22,
 2, 30, :_reduce_23,
 2, 30, :_reduce_24,
 1, 31, :_reduce_none,
 1, 31, :_reduce_26,
 2, 31, :_reduce_27,
 1, 31, :_reduce_28 ]

racc_reduce_n = 29

racc_shift_n = 49

racc_action_table = [
    22,    10,     1,    11,    12,    20,     7,     9,    14,    16,
    18,    39,    40,    35,    36,    37,    22,    22,    22,    45,
    22,    20,    20,    20,    22,    20,    44,    22,    22,    20,
    22,    22,    20,    20,    22,    20,    20,    22,    42,    20,
     6,    34,    20,    14,    16,    18,    30,    14,    16,    18,
    14,    16,    18,     4,     3,    48 ]

racc_action_check = [
    23,     5,     0,     5,     5,    23,     5,     5,     9,     9,
     9,    23,    23,    23,    23,    23,    11,    32,    16,    26,
    36,    11,    32,    16,    25,    36,    25,    10,    33,    25,
    12,    29,    10,    33,    18,    12,    29,    14,    24,    18,
     4,    19,    14,    19,    19,    19,    15,    15,    15,    15,
     7,     7,     7,     2,     1,    43 ]

racc_action_default = [
   -29,   -29,   -29,    -2,   -29,   -29,    49,   -29,    -5,   -29,
   -29,   -29,   -29,    -3,   -29,   -29,   -29,   -11,   -29,   -29,
   -19,   -21,   -18,   -20,   -29,   -29,   -29,    -6,   -16,   -13,
    -9,   -12,   -14,   -15,   -10,   -26,   -29,   -28,   -25,   -23,
   -24,   -22,    -1,   -29,    -4,    -7,   -17,   -27,    -8 ]

racc_action_pointer = [
     0,    51,    53,   nil,    40,    -3,   nil,    39,   nil,    -3,
    24,    13,    27,   nil,    34,    36,    15,   nil,    31,    32,
   nil,   nil,   nil,    -3,    33,    21,    11,   nil,   nil,    28,
   nil,   nil,    14,    25,   nil,   nil,    17,   nil,   nil,   nil,
   nil,   nil,   nil,    47,   nil,   nil,   nil,   nil,   nil ]

racc_goto_table = [
    21,    26,    27,    29,    31,    32,     2,    33,    31,    15,
     8,    19,    25,    38,    13,    43,    24,     5,    23,    46,
    41,   nil,    46,    46,   nil,   nil,    47 ]

racc_goto_check = [
     7,     7,     7,    10,     9,    10,     1,    10,     9,     8,
     6,     8,     5,     7,     4,     7,     3,     2,    11,     7,
    12,   nil,     7,     7,   nil,   nil,     7 ]

racc_goto_pointer = [
   nil,     6,    14,     6,     9,     1,     5,   -10,     2,   -11,
   -11,     8,    -3 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,   nil,   nil,   nil,    28,   nil,    17,
   nil,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :XCLASS => 2,
 :TOKEN => 3,
 :XRULE => 4,
 :XEND => 5,
 :XTOKEN => 6,
 :XSTART => 7,
 :STRING => 8,
 :XPRECHIGH => 9,
 :XPRECLOW => 10,
 :XLEFT => 11,
 :XRIGHT => 12,
 :XNONASSOC => 13,
 ";" => 14,
 ":" => 15,
 "|" => 16,
 "=" => 17,
 :ACTION => 18 }

racc_nt_base = 19

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n ]

Racc_debug_parser = false

##### racc system variables end #####


 # reduce 0 omitted

 module_eval( <<'.,.,', '(build)', 86 )
  def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = val[1].id2name
   result
  end
.,.,

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( <<'.,.,', '(build)', 97 )
  def _reduce_4( val, _values, result )
      @interf.end_register_conv
   result
  end
.,.,

 # reduce 5 omitted

 module_eval( <<'.,.,', '(build)', 103 )
  def _reduce_6( val, _values, result )
      @interf.register_start val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 108 )
  def _reduce_7( val, _values, result )
      @interf.register_conv( val[0], val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 112 )
  def _reduce_8( val, _values, result )
      @interf.register_conv( val[1], val[2] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 117 )
  def _reduce_9( val, _values, result )
      @interf.end_register_prec( true )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 121 )
  def _reduce_10( val, _values, result )
      @interf.end_register_prec( false )
   result
  end
.,.,

 # reduce 11 omitted

 # reduce 12 omitted

 module_eval( <<'.,.,', '(build)', 131 )
  def _reduce_13( val, _values, result )
      @interf.register_prec( :Left, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 135 )
  def _reduce_14( val, _values, result )
      @interf.register_prec( :Right, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 139 )
  def _reduce_15( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 144 )
  def _reduce_16( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 148 )
  def _reduce_17( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 153 )
  def _reduce_18( val, _values, result )
      result = @interf.get_token( result )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 157 )
  def _reduce_19( val, _values, result )
      result = @interf.get_token( eval '"' + val[0] + '"' )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 162 )
  def _reduce_20( val, _values, result )
      unless result.empty? then
        @interf.register_from_array result
      end
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 169 )
  def _reduce_21( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 173 )
  def _reduce_22( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 177 )
  def _reduce_23( val, _values, result )
      unless result.empty? then
        @interf.register_from_array result
      end
      result.clear
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 184 )
  def _reduce_24( val, _values, result )
      pre = result.pop
      unless result.empty? then
        @interf.register_from_array result
      end
      result = [pre]
   result
  end
.,.,

 # reduce 25 omitted

 module_eval( <<'.,.,', '(build)', 195 )
  def _reduce_26( val, _values, result )
      result = OrMark.new( @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 199 )
  def _reduce_27( val, _values, result )
      result = Prec.new( val[1], @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 203 )
  def _reduce_28( val, _values, result )
      result = Action.new( *result )
   result
  end
.,.,

  end   # class RaccParser

end   # module Racc
